#include "main.h"
#include "helper.h"
#include "serial.h"

signed char checkbounds( unsigned int p, char withjitter ) {
	if( withjitter ) {
		if( p <= ROTATEMINVAL + MAXJITTER ) {
			return -1;
		}
		if( p >= ROTATEMAXVAL - MAXJITTER ) {
			return 1;
		}
	} else {
		if( p <= ROTATEMINVAL ) {
			return -1;
		}
		if( p >= ROTATEMAXVAL ) {
			return 1;
		}
	}
	return 0;
}	

unsigned char int2str( unsigned int v, char* buffer, char bufsize )
{
	const unsigned powers[] = { 1u, 10u, 100u, 1000u, 10000u };
	unsigned char i;

	if( !buffer || !bufsize )
		return 0;
	for( i = 1; i < sizeof powers/sizeof powers[0]; i++ ) {
		if (v < powers[i]) {
			break;
		}
	}

	if( bufsize-1 <= i )
		return 0;
	buffer[i] = 0;
	do {
		buffer[--i] = '0' + v%10;
		v /= 10;
	} while (i);

	return 1;
}

unsigned int strlen( char* buffer ) {
	unsigned int res = 0;

	while( buffer[res] != 0 )
		res++;

	return res;
}

unsigned int str2int( char* buffer ) {
	unsigned int len;
	unsigned int i;

	len = strlen( buffer );

	for( i=0; i < len; i++ ) {
		if( !isnum( buffer[i] ) )
			return ERR_INTCONV;
	}

	switch( len ) {
		case 4:
			return (buffer[0]-48) * 1000 + (buffer[1]-48) * 100 + (buffer[2]-48) * 10 + buffer[3]-48;
		case 3:
			return (buffer[0]-48) * 100 + (buffer[1]-48) * 10 + buffer[2]-48;
		case 2:
			return (buffer[0]-48) * 10 + buffer[1]-48;
		case 1:
			return buffer[0]-48;
	}
	return ERR_INTCONV;
}

void memset( char* dest, char val, int len ) {
	unsigned int i = 0;

	while( i < len ) {
		*(dest+i) = val;
		i++;
	}
}

char isnum( char c ) {
	switch( c ) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			return 1;
		default:
			return 0;
	}
}	

// searches for the numth parameter in the string "in" and puts it in "out"
void parseparam( char* in, char* out, char num ) {
	char tmp = 0;

	// if the given string starts with a space, searching for the first word
	while( *in == 32 ) {
		in++;
		if( *in == 0 ) { // no numth parameter
			*out = 0;
			return;
		}
	}

	// searching for the beginning of the numth string
	while( num > 0 ) {
		while( *in != 32 ) {
			if( *in == 0 ) { // no numth parameter
				*out = 0;
				return;
			}
			in++;
		}
		// ok, space found, searching for the beginning of the next word
		while( *in == 32 ) {
			in++;
			if( *in == 0 ) { // no numth parameter
				*out = 0;
				return;
			}
		}
		num--;
	}

	// copying numth parameter to out
	while( *in != 32 && *in != 0 ) {
		out[tmp] = *in;
		in++;
		tmp++;
	}
	out[tmp] = 0;
}

// converts the given string to its uppercase equivalent
void upcase( char* str ) {
	char i = 0;

	while( str[i] != 0 ) {
		switch( str[i] ) {
			case 'q':
			case 'w':
			case 'e':
			case 'r':
			case 't':
			case 'z':
			case 'u':
			case 'i':
			case 'o':
			case 'p':
			case 'a':
			case 's':
			case 'd':
			case 'f':
			case 'g':
			case 'h':
			case 'j':
			case 'k':
			case 'l':
			case 'y':
			case 'x':
			case 'c':
			case 'v':
			case 'b':
			case 'n':
			case 'm':
				str[i] = str[i] - 32;
		}
		i++;
	}
}

void strncpy( char* dest, char* src, char num ) {
	char i = 0;
	while( i < num-1 && src[i] != 0 ) {
		dest[i] = src[i];
		i++;
	}
	dest[i] = 0;
}

// returns 0 if the two given strings match
char strncmp( char* s1, char* s2, char num ) {
	char i = 0;
	while( i < num ) {
		if( s1[i] != s2[i] ) {
			return 1;
		}
		if( s1[i] == 0 ) // if one of the strings -> each string ends
			return 0;
		i++;
	}
	return 2;
}
